
	TITLE	"TV Repeater Timer Logic"

	LIST	P=PIC16C84


#Define	W	0
#Define	F	1

	cblock	0x00
		INDF,RTCC,PCL,Status,FSR,PortA,PortB
	endc
	cblock	0x08
		EEData,EEAdr,PClath,IntCon
	endc

	cblock	0x00
		C,DC,Z,PD,TO,RP0,RP1,IRP
	endc

; Page 1 registers
Roption	EQU	01H
TrisA	EQU	05H
TrisB	EQU	06H
RAMbase	EQU	0CH
PCLATH	EQU	0AH

#define	RXVid	PortB,7  ; receiving video I/P
#define	Acall	PortB,6  ; morse present I/P
#define	SwVid	PortB,5  ; switch video O/P
#define	SwAud	PortB,4  ; switch audio O/P
#define	SetM	PortB,3  ; set morse flag O/P
#define	SetK	PortB,2  ; set K flag O/P
#define	NotSV	PortB,1  ; complement of SwVid O/P
#define	RXlatch	0C,0
#define	Aflag	0C,1
#define	First	0C,2


SKIPCC	MACRO
	BTFSC	Status,C
	ENDM

	LIST

	CBLOCK	RAMbase
		Flags,Count,Count2,Count3,SubSec,Sec	; 6 various registers 
	ENDC

START	ORG	0
	GOTO	Main

INTER	ORG	4
	RETURN

Main
	BSF	Status,RP0	;  adressing bank 1
	MOVLW	B'11000000'
	MOVWF	TrisB		; ports B as outputs except  RB7 and RB6
	MOVLW	B'00000011'	; RTCC set to internal 1:16 and
	MOVWF	Roption		;  we need the integrated pull-up resistors
	BCF	Status,RP0	; adressing bank 0 for the rest of the program
	MOVLW	B'00001110'
	MOVWF	PortB		; set M,K and NotSV outputs at 1

	CLRF	Sec	;Initialisation clock at 0
	CLRF	Count
	CLRF	Count2
	CLRF	Count3
	CLRF	Flags


;Increment time
Time	MOVF	RTCC,W
        BTFSS   Status,Z	;RTCC overflowed (4mS)?
        GOTO    Time		;No
	INCF	SubSec,1	;Increment 1/244th sec
	INCF	RTCC,F
	DECF	Count,F		;count for Acall period to bridge gaps in morse
	MOVLW	-D'244'
	ADDWF	SubSec,W	;Carry now set if second has expired
	SKIPCC			;Zero SubSec if =244
	CLRF	SubSec
	SKIPCC
	INCF	Sec,1		;And increment Seconds
	SKIPCC
	INCF	Count2,1	;And increment count2 squelch open timer
	SKIPCC
	INCF	Count3,1	;And increment count3 closedown timer

;Logic
	BTFSS	Acall		;outputting morse?
	GOTO	Chk_C		;no
	BSF	SetM		;yes-clear flag for morse
	BSF	SetK		;clear flag for K
	BCF	SwAud		;clear repeater mode audio	
	MOVLW	D'40'
	MOVWF	Count		;set timer to bridge gaps in morse
		
Chk_C	MOVF	Count,W	
	BTFSS	Status,Z	;count=0?
	GOTO	Time		;no-go around
	INCF	Count,F		;yes
	BTFSC	RXVid		; receiving ATV signal?
	GOTO	SetSw		;yes
	CLRF	Count2		;no-clear squelch open timer in readiness
	BTFSC	Aflag		;performing morse for closedown?
	GOTO	AudCD		;yes
	BTFSS	RXlatch		;no-has squelch been open for 3 seconds?
	GOTO	ClrRep		;no
	MOVLW	D'10'
	XORWF	Count3,W	;yes-has 10 sec delay expired?
	BTFSS   Status,Z
	GOTO	Time		;no
	BCF	SetK		;yes-send K
	BSF	Aflag		;set flag for closedown morse
	BCF	RXlatch		;clear 3 sec squelch open flag
	CLRF	Sec		;reset 30 sec timer
ClrRep	BCF	SwVid		;switch to beacon mode
	BSF	NotSV
	BCF	SwAud
	GOTO	Time		;go around

SetSw	BSF	SwAud		;switch to repeater mode audio
	BSF	SwVid		;switch to repeater mode video
	BCF	NotSV
	CLRF	Count3		;clear 10 sec delay timer
	BCF	Aflag		;clear closedown flag
	BTFSC	RXlatch		;has squelch been open for 3 seconds?
	GOTO	Time		;yes-go around
	MOVLW	D'3'		;no
	XORWF	Count2,W
	BTFSC	Status,Z	;squelch open timer=3 sec?
	BSF	RXlatch		;yes-set flag
	GOTO	Time		;no-go around

AudCD	MOVLW	D'30'
	XORWF	Sec,W
	BTFSS	Status,Z	;30 sec timer expired?
	GOTO	Time		;no
	CLRF	Sec		;yes-reset timer
	BCF	SetM		;send morse call sign
	BCF	SetK		; followed by K
	BTFSC	First		;first time?
	GOTO	ClrFs		;no
	BSF	First		;yes-set flag
	GOTO	Time

ClrFs	BCF	Aflag		;clear flag for closedown morse
	BCF	First		;clear first flag
	GOTO	Time		

	END
